/* $Id: InspectorDialog.java,v 1.1 2007/10/19 08:19:34 nito Exp $
 * 
 * Copyright (c)ARGO 21, Corporation. 2005, 2006.  All rights reserved.
 * 
 * This file is part of Nautica Workflow.
 * 
 *  Nautica Workflow is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Nautica Workflow; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.tool.wfd;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import jp.co.argo21.nautica.commons.util.ResourceManager;
import jp.co.argo21.nautica.tool.wfd.element.Element;
import jp.co.argo21.nautica.tool.wfd.inspector.AbstractElementInspector;
import jp.co.argo21.nautica.tool.wfd.inspector.ChangeElementListener;
import jp.co.argo21.nautica.tool.wfd.inspector.Inspector;

/**
 * インスペクタ表示用のダイアログを提供する。
 * 
 * @author  Norihiro Itoh(ARGO 21 Corp.)
 * @version $Revision: 1.1 $
 * @since   Nautica Workflow 1.0
 */
public class InspectorDialog extends JDialog
{
	/** シリアルバージョンID */
	private static final long serialVersionUID = 1L;

	/** インペクタマネージャ */
	private static InspectorManager manager;

	/** 表示中のインスペクタ */
	private Inspector inspector;

	/**
	 * インスペクタマネージャの初期化
	 */
	static
	{
		manager = new ElementInspectorManager();
	}

	/**
	 * ダイアログを表示する。
	 * 
	 * @param c	親コンポーネント
	 * @param elem	要素
	 * @return	更新された要素
	 */
	public static Element showDialog(Component c, Element elem)
	{
		Frame frame = JOptionPane.getFrameForComponent(c);
		InspectorDialog dialog = new InspectorDialog(frame, elem);
		dialog.setVisible(true);
		Element result = dialog.inspector.getInspectable();
		dialog.dispose();
		return result;
	}

	/**
	 * ダイアログを表示する。
	 * 
	 * @param c	親コンポーネント
	 * @param elemType	要素型
	 * @return	更新された要素
	 */
	public static Element showDialog(Component c, String elemType)
	{
		Frame frame = JOptionPane.getFrameForComponent(c);
		InspectorDialog dialog = new InspectorDialog(frame, elemType);
		dialog.setVisible(true);
		Element result = dialog.inspector.getInspectable();
		dialog.dispose();
		return result;
	}

	/**
	 * 要素変更リスナーを追加する。
	 * 
	 * @param elemType	要素型
	 * @param l	リスナー
	 */
	public static void addChangeElementListener(String elemType, ChangeElementListener l)
	{
		Inspector i = manager.getInspectorForName(elemType);
		
		if (i instanceof AbstractElementInspector) {
			((AbstractElementInspector)i).addChangeElementListener(l);
		}
	}

	/**
	 * 要素変更リスナーを削除する。
	 * 
	 * @param elemType	要素型
	 * @param l	リスナー
	 */
	public static void removeChangeElementListener(String elemType, ChangeElementListener l)
	{
		Inspector i = manager.getInspectorForName(elemType);
		
		if (i instanceof AbstractElementInspector) {
			((AbstractElementInspector)i).removeChangeElementListener(l);
		}
	}

	/**
	 * ダイアログを生成する。
	 * 
	 * @param frame	親フレーム
	 * @param elem	要素
	 */
	private InspectorDialog(Frame frame, Element elem)
	{
		super(frame, true);

		if (elem == null) {
			inspector = manager.getDefaultInspector();
		} else {
			String inspectorName = ElementInspectorManager.getInspectorKey(elem);
			inspector = manager.getInspectorForName(inspectorName);
		}
		inspector.init();
		inspector.setInspectable(elem);
		
		createDialog();

		setLocationRelativeTo(frame);
	}

	/**
	 * ダイアログを生成する。
	 * 
	 * @param frame	親フレーム
	 * @param elemType	要素型
	 */
	private InspectorDialog(Frame frame, String elemType)
	{
		super(frame, true);

		if (elemType == null) {
			inspector = manager.getDefaultInspector();
		} else {
			inspector = manager.getInspectorForName(elemType);
		}
		inspector.init();
		inspector.setInspectable(null);
		
		createDialog();

		setLocationRelativeTo(frame);
	}

	/**
	 * ダイアログを作成する。
	 */
	private void createDialog()
	{
		ResourceManager rm = WorkflowDesignerManager.getResourceManager();

		setTitle(rm.getResource("InspectorDialog.title"));
		
		JScrollPane scroll = new JScrollPane();
		scroll.setViewportView((JComponent)inspector);

		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT, 10, 10));

		ActionListener listener = new CloseAction();
		if (inspector.isAutoCommit()) { 
			JButton closeButton = new JButton(rm.getResource("JButton.label.close"));
			closeButton.addActionListener(listener);
			getRootPane().setDefaultButton(closeButton);
			buttonPane.add(closeButton);
		} else {
			JButton setButton = new JButton(rm.getResource("JButton.label.set"));
			setButton.setActionCommand("OK_OPTION");
			setButton.addActionListener(listener);
			getRootPane().setDefaultButton(setButton);
			buttonPane.add(setButton);

			JButton cancelButton = new JButton(rm.getResource("JButton.label.cancel"));
			cancelButton.addActionListener(listener);
			buttonPane.add(cancelButton);
		}

		Container contentPane = getContentPane();
		contentPane.add(scroll, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);

		pack();
	}

	/**
	 * ダイアログが閉じたときの処理
	 */
	private class CloseAction implements ActionListener
	{
		/**
		 * 検証を行ってから閉じる。
		 * 
		 * @param e	イベント
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e)
		{
			if ("OK_OPTION".equals(e.getActionCommand())) {
				if (inspector.check() == false) {
					//エラーがあったので、コミットせずにreturn
					//ダイアログは閉じない
					return;
				}
				inspector.commit();
			}
			inspector.unload();
			setVisible(false);
		}
	}
}
